#!/bin/sh

echo 2 > /proc/sys/vm/overcommit_memory
echo 100 > /proc/sys/vm/overcommit_ratio

start_lock=0;
data_file=/tmp/dm_tmp_file
MODE="auto"

#some validation
if [ -f "./fmw_board_type" ]; then
    cur_board=`factory board | sed y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/ | sed s/-//g`

    if [ "$cur_board" != "VP15PMGF" ]; then
        cur_board=`echo "$cur_board" | grep -o ^[A-Z]*[0-9]*`
    fi

    case "$cur_board"  in
    "VP12P")      board_type="VP12P";    start_lock=1;;
    "VP12")       board_type="VP12";     start_lock=1;;
    "VP15P")      board_type="VP15P";    start_lock=1;;
    "VP15")       board_type="VP15";     start_lock=1;;
    "VP15PMGF")   board_type="VP15PMGF"; start_lock=1;;
    "VP-15P-MGF") board_type="VP15PMGF"; start_lock=1;;
    esac

    if [ -z "$board_type" ]; then
        echo "Upgrade firmware FAIL: wrong factory settings" > /dev/console
        exit 1;
    fi

    fmw_type=`cat ./fmw_board_type`
    if [ "$fmw_type" != "$board_type" ]; then
        echo "Upgrade firmware FAIL: firmware for $fmw_type, but board is $cur_board" > /dev/console
        exit 1;
    fi
else
    echo "Upgrade firmware FAIL: unknown board type" > /dev/console
fi

if [ -f /tmp/fw/utils/upgrade-bin ]; then
    FW_PATH="/tmp/fw"
    MODE="manual"
else
    if [ -f /tmp/auto_fw_upgrade/utils/upgrade-bin ]; then
        FW_PATH="/tmp/auto_fw_upgrade"
    else
        if [ -f /tmp/dhcp_fw_upgrade/utils/upgrade-bin ]; then
            FW_PATH="/tmp/dhcp_fw_upgrade"
        else
            FW_PATH="/tmp/fw_tr069"
        fi
    fi
fi

md5_cur=`busybox md5sum ./linux.bin | cut -d' ' -f 1`
md5_img=`cat md5_linux`
if [ "$md5_cur" != "$md5_img" ]; then
    echo "Upgrade firmware FAIL: incorrect CRC" > /dev/console
    exit 1;
fi

soft_major=$(cat $FW_PATH/version | cut -d'.' -f 1)
soft_minor=$(cat $FW_PATH/version | cut -d'.' -f 2)
hw_major=$(factory hw | sed "s/V/./g" | sed "s/v/./g" | cut -d'.' -f 1)
hw_minor=$(factory hw | sed "s/V/./g" | sed "s/v/./g" | cut -d'.' -f 2)
if [ "$hw_major" -ge "2" ] && [ "$soft_major" -eq "1" ] && [ "$soft_minor" -lt "4" ]; then
    echo "Upgrade firmware FAIL: on board revision $hw_major.$hw_minor cannot use software version $soft_major.$soft_minor" > /dev/console
    exit 4;
fi

if [ -f /tmp/have_active_calls ]; then
    if [ "$MODE" == "manual" ]; then
        echo "Active calls detected, try upgrade firmware again later" > /dev/console
        exit 5;
    else
        if [ ! -f /tmp/waiting_for_upgrade ]; then
            echo "Active calls detected, upgrade firmware will continue after calls are completed" > /dev/console
            echo "" > /tmp/waiting_for_upgrade
            while [ -f /tmp/have_active_calls ]; do
                sleep 1
            done
            echo "Continue upgrade firmware after waiting active calls" > /dev/console
            rm -f /tmp/waiting_for_upgrade
        else
            echo "Upgrade firmware is already scheduled" > /dev/console
            exit 0;
        fi
    fi
fi

if [ $start_lock = 1 ]; then
    ps|grep driver_manager > $data_file
    data2=$(sed -n 's/run_daemon//p' $data_file);
    count=$(echo ${#data2})
    if [ $count = 0 ];
        then
            data2=$(sed -e  '2,$d' $data_file);
        else
            data2=$(sed -e  '1d;3,$d' $data_file);
    fi
    pid=$(echo $data2|sed -e 's/root.*//;s///')
    kill -s 17 $pid
fi

killall igmpproxy udpxy

chmod 777 $FW_PATH/utils/stop_voip
$FW_PATH/utils/stop_voip
chmod 777 $FW_PATH/upgrade-bin

echo "Stopping VoIP service" > /dev/console
kill `ps| grep "/bin/run_daemon.sh voip"|head -n 1|cut -d' ' -f 2`
echo "" > /tmp/restart_voip

sleep 3
sleep 3
killall voip
sleep 3

cur_major=$(cat /version | cut -d'.' -f 1)
cur_minor=$(cat /version | cut -d'.' -f 2)

#Workaround for 1.5.0
if [ $cur_major -eq 1 ]; then
    if [ $cur_minor -eq 5 ]; then
        mv /bin/factory /bin/_factory
    fi
fi

echo "Upgrade firmware STARTED" > /dev/console
$FW_PATH/utils/upgrade-bin ./linux.bin &> /dev/console
RETVAL=$?

if [ "$RETVAL" != "0" ]; then
    if [ "$RETVAL" = "254" ]; then
        echo "Upgrade firmware FAIL: upgrade process is already in progress" > /dev/console
        exit 2;
    fi
    echo "Upgrade firmware FAIL: Error at read/write operation" > /dev/console
    exit 1;
fi

if [ $cur_major -eq 1 ]; then
    if [ $cur_minor -eq 5 ]; then
        mv /bin/_factory /bin/factory
    fi
fi

if [ -e "$FW_PATH/defaults" ];then
    echo "Upgrade firmware: update def.yaml" > /dev/console;
    cd $FW_PATH
    cat /proc/mtd | grep "mtd2" > /dev/null
    if [ $? == 0 ]; then
        tar -c defaults | gzip | $FW_PATH/utils/flash_util default_config
    else
        tar -c defaults | gzip | $FW_PATH/utils/save_def_yaml_old
    fi
#    tar -c defaults | gzip | $FW_PATH/utils/flash_util default_config
    cd -
fi

bvesion="0"
if [ -f "/bin/boot_version" ];then
res=`/bin/boot_version`
  if [ "$res" != "" ];then
        bversion="1"
  fi
fi

if [ "$bversion" == "1" ];then
    bank="`cat /proc/bootbank`"
    if [ "$bank" == "1" ];then
        md5sum set linux2 `cat $FW_PATH/md5_linux`
    else
        md5sum set linux1 `cat $FW_PATH/md5_linux`
    fi
else
    echo "Old loader" > /dev/console
fi

echo "Firmware upgrade SUCCESSFULLY completed" > /dev/console
exit 0;
